require('dotenv').config();
const request = require('./requests');
const helper = require('./helper');
const fs = require('fs-extra');
const extract = require('extract-zip');
const core = require('@actions/core');

const __parentDirName = require('path').resolve(__dirname, '..');
const latestPackageUrl = process.env.MIMO_TEACH_URL + 'packages/latest';
const latestVersion = Number(
  fs.readFileSync('./.package-version', 'utf8').trim(),
);

async function authenticate() {
  let { body } = await request.post('https://mimo.auth0.com/oauth/ro', {
    headers: {
      'Content-Type': 'application/json',
    },
    body: helper.oauthBody(),
  });

  const responseBody = JSON.parse(body);
  console.log(responseBody);
  bearerToken = 'Bearer ' + responseBody.id_token;
  return bearerToken;
}

async function getLatestPackageInfo(bearerToken) {
  // Updates the Lessons repo if the latest version
  // number is different than the last time the script was run.
  let { body } = await request.get(latestPackageUrl, {
    headers: helper.alexandriaHeaders(bearerToken),
  });

  console.log({ body });
  const responseBody = JSON.parse(body);
  const downloadPackageUrl = responseBody.downloadUrl;
  const newVersion = responseBody.version;
  const packagedAt = responseBody.packagedAt;
  const publishedAt = responseBody.publishedAt;

  console.log(responseBody);
  console.log({
    latestVersion,
  });
  return { newVersion, downloadPackageUrl, packagedAt, publishedAt };
}

async function downloadZip(downloadPackageUrl) {
  const zipFile = __parentDirName + '/tracks.zip';

  return new Promise((resolve, reject) =>
    request.download(downloadPackageUrl, zipFile).on('finish', function() {
      extract(
        __parentDirName + '/tracks.zip',
        {
          dir: __parentDirName + '/TempTracks/',
        },
        async function(err) {
          if (err) {
            reject(err);
          }
          resolve();
        },
      );
    }),
  );
}

async function run() {
  try {
    let bearerToken = await authenticate();
    console.log({ bearerToken });

    let {
      newVersion,
      downloadPackageUrl,
      packagedAt,
      publishedAt,
    } = await getLatestPackageInfo(bearerToken);

    if (latestVersion !== newVersion) {
      await downloadZip(downloadPackageUrl);
      helper.updateContentVersionNumber(newVersion);
      core.exportVariable(
        'PULL_REQUEST_TITLE',
        `Update content package to version ` + newVersion,
      );
      core.exportVariable(
        'PULL_REQUEST_BRANCH',
        `content-update-` + newVersion,
      );
      core.exportVariable(
        'PULL_REQUEST_DESCRIPTION',
        `Update content package to: \nversion: ${newVersion}\npackagedAt: ${packagedAt}\npublishedAt: ${publishedAt}`,
      );
      helper.replaceOldTrackDirectories(__parentDirName);
    } else {
      // No need to do anything when the current content already exists on Github
      console.log('the package is already up to date');
    }
  } catch (error) {
    console.log(error);
  }
}

run();
